import datetime


class AbstractSerializerService:
    """The serializer class that will handle the formatting of data to readable formats:
    ie: json, dict, List.
    """

    # The list of field enlisted for serialization.
    __serializable__: list = None

    # The list of default values regardless of the object defined.
    __default__fields__: list = None

    def __init__(self):
        if self.__serializable__ is None:
            raise NotImplementedError('Serializable attribute is None, serializable fields must be defined.')

        if self.__default__fields__ is None:
            self.__default__fields__ = []

    def __get_serializable(self):
        return self.__serializable__ + self.__default__fields__

    @staticmethod
    def __to_date(date: datetime.datetime, date_only=False) -> str:
        """Date.date obj converter, returns string value of formatted datetime.datetime obj.

        note: We will mode this method to gwx-core/utils package

        :param date: The datetime.datetime object
        :param date_only: bool flag, decides if the return value is only date or includes it's timestamp
        :return:
        """
        return str(date).split('.')[0]

    def to_json(self) -> dict:
        """Converts the obj to a serialized dict obj as {key: value}.

        :return: serialized attributes as dict obj
        """
        result = {}
        filters = self.__serializable__ + self.__default__fields__

        for key in dir(self):
            if key in filters:
                value = self.__getattribute__(key)

                result.update([(key, value)])

                if type(value) == datetime.datetime:
                    result.update([(key, self.__to_date(value))])

        return result



from typing import Tuple, Any
from unittest import TestCase


class UtilsTestCase(TestCase):
    """An extended instance of TestCase specific for utils modules.

    Contains test-cases for the following modules:
        - response
        - extractor
    """

    def assertRaiseWithMessageOnResponse(
            self,
            expected_exception: BaseException or Tuple[BaseException, Any],
            expected_fail_method: callable,
            exception_message: str,
            message: str or None,
            data: dict or None,
            headers: dict or None,
            no_parameters=True
    ):
        """
        Assert the raised exception to be the expected exception provided,
        assert the exception message is equal to the exception message provided.


        :param expected_exception: instance of BaseException
        :param expected_fail_method: the function()/method() that will cause the exception error
        :param exception_message: the exception message specified
        :param message: the message that is set on the response
        :param data: the data that is set on the response
        :param headers: the headers that is set on the response
        :param no_parameters:
        :return: None
        """
        with self.assertRaises(expected_exception) as context:

            if no_parameters:
                expected_fail_method(message, data, headers)
            else:
                expected_fail_method(expected_fail_method)

        self.assertEqual(exception_message, str(context.exception))



import os
import re


def file_name_conversion(file_path: str, extension_to_remove=None) -> str:
    """Convert a `/path/to/file.py` to it's usable string name such as `file`.

    :param file_path: the absolute path of a file preferable the __file__ constant.
    :param extension_to_remove: the string that you find irrelevant and needs to be removed
    ie: `/path/to/file_extension.py`, specifying `_extension` will only return `file`
    :return: converted str stripped
    """

    try:
        open(file_path)
    except FileNotFoundError:
        raise FileNotFoundError(f'Error file: {file_path} does not exists.')

    name, extension = os.path.splitext(os.path.basename(file_path))

    if extension_to_remove:
        return str.lower(str(name.split(extension_to_remove)[0]))

    return str.lower(str(name))


def pascal_case(word: str) -> str:
    """
    Convert passed words to pascal case, ie:

    - pascal_case("San Francisco")  # SanFrancisco
    - pascal_case("SAN-FRANCISCO")  # SanFrancisco
    - pascal_case("san_francisco")  # SanFrancisco

    :param word: passed word as string
    :return: the converted string
    """
    word_regex_pattern = re.compile("[^A-Za-z]+")
    words = word_regex_pattern.split(word)
    return "".join(w.title() if i is 0 else w.title() for i, w in enumerate(words))
